# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"OLD2NEW_LABELS_DICT": "01a_constants.ipynb",
         "OLD_LABELS": "01a_constants.ipynb",
         "NEW_LABELS": "01a_constants.ipynb",
         "MAX_VALUES_BY_DTYPE_STR": "01a_constants.ipynb",
         "URL": "01a_constants.ipynb",
         "BAND_STATS": "01a_constants.ipynb",
         "BAND_STATS_FLOAT32": "01a_constants.ipynb",
         "BEN_CHANNELS": "01a_constants.ipynb",
         "BEN_10m_CHANNELS": "01a_constants.ipynb",
         "BEN_20m_CHANNELS": "01a_constants.ipynb",
         "BEN_10m_20m_CHANNELS": "01a_constants.ipynb",
         "BEN_30m_CHANNELS": "01a_constants.ipynb",
         "BEN_RGB_CHANNELS": "01a_constants.ipynb",
         "COUNTRIES": "01a_constants.ipynb",
         "COUNTRIES_ISO_A2": "01a_constants.ipynb",
         "print_all_constants": "01a_constants.ipynb",
         "USER_DIR": "01b_base.ipynb",
         "parse_datetime": "01b_base.ipynb",
         "box_from_ul_lr_coords": "01b_base.ipynb",
         "PATCHES_WITH_SNOW_URL": "01b_base.ipynb",
         "PATCHES_WITH_CLOUD_AND_SHADOW_URL": "01b_base.ipynb",
         "get_patches_with_seasonal_snow": "01b_base.ipynb",
         "get_patches_with_cloud_and_shadow": "01b_base.ipynb",
         "is_snowy_patch": "01b_base.ipynb",
         "is_cloudy_shadowy_patch": "01b_base.ipynb",
         "patches_from_original_train_split": "01b_base.ipynb",
         "patches_from_original_validation_split": "01b_base.ipynb",
         "patches_from_original_test_split": "01b_base.ipynb",
         "get_original_split_from_patch_name": "01b_base.ipynb",
         "old2new_labels": "01b_base.ipynb",
         "COUNTRIES_URL": "01c_gdf_builder.ipynb",
         "ben_patch_to_gdf": "01c_gdf_builder.ipynb",
         "ben_patch_to_reprojected_gdf": "01c_gdf_builder.ipynb",
         "build_gdf_from_patch_paths": "01c_gdf_builder.ipynb",
         "get_patch_directories": "01c_gdf_builder.ipynb",
         "get_gdf_from_patch_dir": "01c_gdf_builder.ipynb",
         "get_ben_countries_gdf": "01c_gdf_builder.ipynb",
         "assign_to_ben_country": "01c_gdf_builder.ipynb",
         "Season": "01c_gdf_builder.ipynb",
         "tfm_month_to_season": "01c_gdf_builder.ipynb",
         "filter_season": "01c_gdf_builder.ipynb",
         "add_full_ben_metadata": "01c_gdf_builder.ipynb",
         "remove_bad_ben_gdf_entries": "01c_gdf_builder.ipynb",
         "build_raw_ben_parquet": "01c_gdf_builder.ipynb",
         "extend_ben_parquet": "01c_gdf_builder.ipynb",
         "remove_discouraged_parquet_entries": "01c_gdf_builder.ipynb",
         "build_recommended_parquet": "01c_gdf_builder.ipynb",
         "recurse_link_files": "01d_subset_builder.ipynb"}

modules = ["constants.py",
           "base.py",
           "gdf_builder.py",
           "subset_builder.py"]

doc_url = "https://k.clasen.github.io/bigearthnet_common/"

git_url = "https://github.com/k.clasen/bigearthnet_common/tree/main/"

def custom_doc_links(name): return None
